/* Condensation Graph
 * 
 * Reducere la Directed Acyclic Graph (DAG)
 * Colapsarea componentelor tare conexe in noduri.
 * Se obtine un nou graf orientat aciclic,
   numit Graf Condensat (Condensed Graph)
 */
#include <iostream>
#include <fstream>
#include <vector>
#include <algorithm>
#include <stack>
using namespace std;

ifstream fin("graf.in");
ofstream fout("graf.out");

using VI  = vector<int>;
using VVI = vector<VI>;
using VB  = vector<bool>;

VVI G;
VVI dag;         // listele de adiacenta pt dag
VI  lv, low;
VI c;            // c[x] = indicele componentei tare-conexe de care apartine nodul x
stack<int> stk;  // stiva din care extragem ctc
VB inStack, v;

int n, N;       // n graful initital, N - nr noduri dag
int idx;
int cnt;        // comp conexa curenta

void CitesteGraf();
void Tarjan(int x);
void CreazaDAG();
void ScrieDag();

int main()
{
	CitesteGraf();
	for (int i = 1; i <= n; ++i)
		if (lv[i] == 0)
			Tarjan(i);

	CreazaDAG();
	ScrieDag();
}

int y;

void Tarjan(int x)
{
	low[x] = lv[x] = ++idx;
	stk.push(x);
	inStack[x] = true;

	for (const int& y : G[x])
		if (lv[y] == 0)
		{
			Tarjan(y);
			low[x] = min(low[x], low[y]);
		}
		else
			if (inStack[y])
				low[x] = min(low[x], lv[y]);

	if (low[x] == lv[x])
	{
		N++;  // o noua componenta tare-conexa
		while (true)
		{
			y = stk.top();
			c[y] = N;      // comp lui y este N
			stk.pop();
			inStack[y] = false;
			if (y == x)
				break;
		}
	}
}

void CreazaDAG()  // arcele intre componentele tare-conexe
{
	dag = VVI(N + 1);
	for (int x = 1; x <= n; ++x)
		for (const int& y : G[x])
			if (c[x] != c[y])
				dag[c[x]].push_back(c[y]);
}

void ScrieDag()
{
	fout << "N = " << N << '\n';
	for (int comp = 1; comp <= N; ++comp)
	{
		fout << x << " : ";
		for (const int& x : dag[comp])
			fout << x << ' ';
		fout << '\n';
	}

	fout << "Ce contin nodurile DAG:\n";
	for (int i = 1; i <= N; ++i)
	{
		fout << "Nodul " << i << " : ";
		for (int j = 1; j <= n; ++j)
			if (c[j] == i)
				fout << j << ' ';
		fout << '\n';
	}
}

void CitesteGraf()
{
	fin >> n;
	G = VVI(n + 1);
	lv = low = VI(n + 1);
	inStack = v = VB(n + 1);
	c = VI(n + 1);

	for (int x, y; fin >> x >> y;
		G[x].push_back(y));
}

