/**
    Condensation Graph
    Colapsarea componentelor tare-conexe in noduri
    Se obtine un graf orientat aciclic (Directed Acyclic Graph) - DAG
*/
#include <fstream>
#include <vector>
#include <stack>
#include <set>
using namespace std;
using VB  = vector<bool>;
using VI  = vector<int>;
using VVI = vector<VI>;
using SI  = set<int>;

ifstream fin("dag.in");
ofstream fout("dag.out");

int n, m;
VVI G, Gt;

VI c;     /// c[x] = componenta tare conexa din care face parte x
int N;    /// nr de noduri ale grafului condensat
stack<int> stk;  /// retinem nodurile in ordine topologica
VB inStack;
VB v;
VVI dag;  /// retine graful condensat

void CitesteGraf();
void Dfs(int x);
void DfsT(int x);
void Kosaraju();
void CreazaDAG();
void ScrieDAG();

int main()
{
    CitesteGraf();
    Kosaraju();
    CreazaDAG();
    ScrieDAG();
}

void Dfs(int x)
{
    v[x] = true;
    for (int y : G[x])
    {
        if (v[y]) continue;
        Dfs(y);
    }
    stk.push(x);
}

void DfsT(int x)
{
    v[x] = true;
    c[x] = N;
    for (int y : Gt[x])
    {
        if (v[y]) continue;
        DfsT(y);
    }
}

void Kosaraju()
{
    v = VB(n + 1);
    for (int x = 1; x <= n; ++x)
        if (!v[x])
            Dfs(x);

    v = VB(n + 1);
    while (!stk.empty())
    {
        int x = stk.top();
        stk.pop();
        if (v[x]) continue;
        N++;
        DfsT(x);
    }
}

void ExtractCTC(int x)
{
    N++;
    while (true)
    {
        int y = stk.top();
        stk.pop();
        inStack[y] = false;
        c[y] = N;
        if (y == x)
            break;
    }
}

void CreazaDAG()
{
    dag = VVI(N + 1);
    for (int x = 1; x <= n; ++x)
        for (int y : G[x])
            if (c[x] != c[y])
                dag[c[x]].push_back(c[y]);
}

void CitesteGraf()
{
    fin >> n >> m;
    G = Gt = VVI(n + 1);
    inStack = VB(n + 1);
    c = VI(n + 1);
    int x, y;
    while (m--)
    {
        fin >> x >> y;
        G[x].push_back(y);
        Gt[y].push_back(x);
    }
}

void ScrieDAG()
{
    for (int x = 1; x <= N; ++x) // pt fiecare nod din DAG
    {
        for (int y : dag[x])
            fout << y << ' ';
        fout << '\n';
    }
}


