	
#include <iostream>
#include <fstream>
#include <vector>
#include <stack>
using namespace std;
 
ifstream fin("dep.in");
ofstream fout("dep.out");
 
const int Mod = 666013;
 
using VI = vector<int>;
using VVI = vector<VI>;
using VB = vector<bool>;
using VVB = vector<VB>;
 
int N, M, K;
VVI G0, G;
VVI comp;			// comp[i] = nodurile cuprinse de componenta tare-conexa i
VI ctc;				// ctc[nod] = din ce comp. tare-conexa face parte nodul i
VI t;
 
int nr;
VI low, idx;
stack<int> st;
VB inStack;
int C;
 
VVI D;
VI smax;
 
void ReadData();
void TarjanAlg(int node);
void Solve();
void DFS(int node);		// DFS pentru graful/arborii format/i din ctc ca noduri
void Add(int& a, int b);
 
int main()
{
	ReadData();
	Solve();
	
	fin.close();
	fout.close();
	return 0;
}
 
void ReadData()
{
	fin >> N >> M >> K;
	G0 = VVI(N + 1);
	
	int x, y;
	while (M--)
	{
		fin >> x >> y;
		
		G0[x].push_back(y);
	}
}
 
void TarjanAlg(int node)
{
	low[node] = idx[node] = ++nr;
	st.push(node); inStack[node] = true;
	
	for (const int& next : G0[node])
	{
		if (!idx[next])
		{
			TarjanAlg(next);
			low[node] = min(low[node], low[next]);
		}
		else
			if (inStack[next])
				low[node] = min(low[node], low[next]);
	}
	
	if (low[node] == idx[node])
	{
		int n;
		VI c;
		
		do
		{
			n = st.top();
			st.pop(); inStack[n] = false;
			
			c.push_back(n);
			ctc[n] = comp.size();
		}while (n != node);
		
		comp.push_back(c);
	}
}
 
void Solve()
{
	ctc = low = idx = VI(N + 1);
	inStack = VB(N + 1);
	
	for (int i = 1; i <= N; ++i)
		if (!idx[i])
			TarjanAlg(i);
 
	// trebuie sa fac reducerea tranzitiva
	G = VVI(comp.size());
	t = VI(comp.size());
	VVB m(comp.size(), VB(comp.size()));
	VI gi(comp.size());
	
	// m - matrice de adiacenta
	for (int i = 1; i <= N; ++i)
		for (const int& v : G0[i])
			if ( ctc[i] != ctc[v] )
			{
				m[ctc[i]][ctc[v]] = true;
			}
	
	C = (int)comp.size();
	// Floyd Warshall => Matricea drumurilor
	for (int k = 0; k < C; ++k)
		for (int i = 0; i < C; ++i)
			for (int j = 0; j < C; ++j)
				if (i != j && m[i][k] && m[k][j])
					m[i][j] = true;
	
	for (int i = 0; i < C; ++i)
		for (int j = 0; j < C; ++j)
			if (m[i][j])
			{
				for (int k = 0; k < C; ++k)
					if (m[j][k])
						m[i][k] = false;
			}
	
	// G[j] - toate nodurile (comp tare conexe) din care pot ajunge in j
	for (int i = 0; i < C; ++i)
		for (int j = 0; j < C; ++j)
			if (m[i][j])
			{
				G[j].push_back(i);
				++gi[i];   // grad interior
			}
	
	D = VVI(C, VI(N + 1));
	smax = VI(C);
	
	VI s(N + 1);
	int sm {0};
	s[0] = 1;
	
	for (int i = 0; i < C; ++i)
	{
		if (gi[i] == 0) // radacina i a fiecarui arbore
		{
			DFS(i);

			// rucsac
			for (int j = sm; j >= 0; --j)
				for (int sum = 1; sum <= smax[i]; ++sum)
					Add(s[j + sum], (1LL * s[j] * D[i][sum]) % Mod);
			
			sm += smax[i];
		}
	}
	
	fout << s[K];
}
 
void DFS(int node)
{
	D[node][0] = 1;
	smax[node] = comp[node].size();
	
	for (const int& son : G[node])
	{
		DFS(son);
		for (int i = smax[node]; i >= 1; --i)
			for (int j = 1; j <= smax[son]; ++j)
				Add(D[node][i + j], (1LL * D[node][i] * D[son][j]) % Mod);
			
			
		for (int j = 1; j <= smax[son]; ++j)
			Add(D[node][comp[node].size() + j], D[son][j]);
		
		smax[node] += smax[son];
	}
	
	Add(D[node][comp[node].size()], 1);
}
 
void Add(int& a, int b)
{
	a = (a + b) % Mod;
}
