// AC 1.39
// https://www.spoj.com/problems/GOODA/
#include <iostream>
#include <cstdio>
#include <numeric>
#include <algorithm>
#include <vector>
#include <queue>
#include <stack>
using namespace std;
using VB  = vector<bool>;
using VI  = vector<int>;
using VLL = vector<long long>;
using VVI = vector<VI>;

int S, E;
int N;
VI fun, c;             /// c[x] indicele componentei tare-conexe care contine nodul x
VB v;
VLL fun_ctc, max_fun;  /// fun_ctc[k] = suma valorilor fun din comp tare-conexa k
                       /// max_fun[k] - suma MAXIMA a valorilor fun pe un traseu care se termina cu componenta tare-conexa k
VVI G, Gt, dag;
stack<int> stk;        /// retine nodurile in ordine totpologica
int n, m;

void CitesteGraf();
void Kosaraju();
void Dfs(int x);
void DfsT(int x);
void CreazaDAG();
void BellmanFord();

int main()
{
    CitesteGraf();
	Kosaraju();
    CreazaDAG();
    BellmanFord();

    cout << max_fun[c[E]] << '\n';

    return 0;
}

void BellmanFord()  /// Bellman Ford pentru distante maxime (max_fun)
{
    queue<int> Q;
	VB inQ(N + 1);
	max_fun[c[S]] = fun_ctc[c[S]]; /// luam toate valorile din componenta de plecare
    Q.push(c[S]);
    inQ[c[S]] = true;

    while (!Q.empty())
    {
        int i = Q.front();  // i, j = nr de ordine a doua componente t-c
        Q.pop();
        inQ[i] = false;
        for (int j : dag[i])
            if (max_fun[j] < max_fun[i] + fun_ctc[j])
            {
                max_fun[j] = max_fun[i] + fun_ctc[j];
                if (!inQ[j])
                {
                    Q.push(j);
                    inQ[j] = true;
                }
            }
        }
}

void Kosaraju()
{
    v = VB(n + 1);
    for (int x = 1; x <= n; ++x)
    {
        if (v[x]) continue;
        Dfs(x);
    }

    v = VB(n + 1);

    while (!stk.empty())
    {
        int x = stk.top();
        stk.pop();
        if (v[x]) continue;
        N++;     // o noua componenta tare-conexa
        DfsT(x);
    }
}

void CreazaDAG()
{
	dag = VVI(N + 1);
    for (int x = 1; x <= n; ++x)
        for (int y : G[x])
            if (c[x] != c[y])
                dag[c[x]].push_back(c[y]);
}

void Dfs(int x)
{
    v[x] = true;
    for (int y : G[x])
        if (!v[y])
            Dfs(y);
    stk.push(x);
}

void DfsT(int x)
{
    v[x] = true;
    c[x] = N;
    fun_ctc[N] += fun[x];
    for (int y : Gt[x])
        if (!v[y])
            DfsT(y);
}

void CitesteGraf()
{
    ios_base::sync_with_stdio(0);
    cin.tie(0);
    cin >> n >> m;
    G = Gt = VVI(n + 1);
    fun = c = VI(n + 1);
    fun_ctc = max_fun = VLL(n + 1);

    cin >> S >> E;
    for (int i = 1; i <= n; i ++)
        cin >> fun[i];
	int x, y;
    for (int i = 0; i < m; ++i)
    {
        cin >> x >> y;
        G[x].push_back(y);
        Gt[y].push_back(x);
    }
}
