
#include <iostream>
#include <algorithm>
#include <vector>
#include <queue>
#include <stack>
using namespace std;
using VB  = vector<bool>;
using VI  = vector<int>;
using VLL = vector<long long>;
using VVI = vector<VI>;

int S, E;
int N;                 /// nr de comp tare conexe
VI fun, c;             /// c[x] = indicele componentei tare-conexe de care apartine nodul x
VLL fun_ctc, max_fun;  /// fun[k] = suma valorilor fun din comp tare-conexa k
                       /// max_fun[k] - suma MAXIMA a valorilor fun pe un traseu care se termina cu componenta tare-conexa k
VVI G, dag;
stack<int> stk;        /// retine nodurile in ordinea descoperirii lor la
VB inStack; /// inStack[x] = true, daca x este in stiva
VI index;   /// index[x] = indexul (timpul de descoperire a lui x)
VI low;     /// low[x] = indexul minim a care se poate
            /// pornind din nodul x si folosind o singura muchie de intoarcere
int idx;    /// indexul curent
int n, m;

void CitesteGraf();
void Tarjan();
void Dfs(int x);
inline void ExtractCTC(int x);
void CreazaDAG();
void BellmanFord();

int main()
{
    CitesteGraf();
	Tarjan();
    CreazaDAG();
    BellmanFord();

    cout << max_fun[c[E]] << '\n';

    return 0;
}

void Tarjan()
{
    for (int x = 1; x <= n; ++x)
        if (index[x] == 0)
            Dfs(x);
}

void Dfs(int x)
{
	index[x] = low[x] = ++idx;
    stk.push(x);
    inStack[x] = true;

    for (int y : G[x])
        if (!index[y])  // nod nevizitat, deci y este fiul lui x
        {
            Dfs(y);
            low[x] = min(low[x], low[y]);
        }
        else                 // nod vizitat
            if (inStack[y])  // y - stramos
                low[x] = min(low[y], index[y]);

    if (low[x] == index[x])
        ExtractCTC(x);
}

inline void ExtractCTC(int x)  /// comp tare-conexa curenta
{
    N++;
    while (!stk.empty())
    {
        int y = stk.top();
        stk.pop();
        inStack[y] = false;
        c[y] = N;       /// componenta t-c a lui x este N
        fun_ctc[N] += fun[y];
        if (y == x)
            break;
    }
}

void BellmanFord()  /// Bellman Ford pentru distante maxime (max_fun)
{
    queue<int> Q;
	VB inQ(N + 1);
	max_fun[c[S]] = fun_ctc[c[S]];  /// luam toate valorile din componenta de plecare
    Q.push(c[S]);
    inQ[c[S]] = true;

    while(!Q.empty())
    {
        int i = Q.front();
        Q.pop();
        inQ[i] = false;
        for (int j : dag[i])  // i si j = indici a doua componete t-c
            if (max_fun[j] < max_fun[i] + fun_ctc[j])
            {
                max_fun[j] = max_fun[i] + fun_ctc[j];
                if (!inQ[j])
                {
                    Q.push(j);
                    inQ[j] = true;
                }
            }
        }
}

void CreazaDAG()
{
	dag = VVI(N + 1);
    for (int x = 1; x <= n; ++x)
        for (int y : G[x])
            if(c[x] != c[y])
                dag[c[x]].push_back(c[y]);
}

void CitesteGraf()
{
    ios_base::sync_with_stdio(0);
    cin.tie(0);
    cin >> n >> m;
    G = VVI(n + 1);
    fun = c = index = low = VI(n + 1);
    fun_ctc = max_fun = VLL(n + 1);
    inStack = VB(n + 1);

    cin >> S >> E;
	for (int i = 1; i <= n; ++i)
		cin >> fun[i];
	int x, y;
    for (int i = 1; i <= m; ++i)
    {
        cin >> x >> y;
        G[x].push_back(y);
    }
}
